//  
//  Copyright (C) 2010 Robert Dyer
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 
using System;

using Docky.Items;

namespace RecentDocuments
{
	public class EmptyRecentItem : IconDockItem
	{
		#region AbstractDockItem implementation
		
		public override string UniqueID ()
		{
			return "EmptyRecentItem";
		}
		
		#endregion
		
		public EmptyRecentItem ()
		{
			Icon = "folder-recent;;document-open-recent";
			HoverText = "Recent Documents";
		}
	}
}
