//  
//  Copyright (C) 2009 Jason Smith, Robert Dyer
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using System;
using System.Collections.Generic;
using System.Linq;

namespace Docky.Items
{
	public class ItemsChangedArgs : EventArgs
	{
		public IEnumerable<AbstractDockItem> AddedItems { get; protected set; }
		public IEnumerable<AbstractDockItem> RemovedItems { get; protected set; }

		public ItemsChangedArgs ()
		{
			AddedItems = Enumerable.Empty<AbstractDockItem> ();
			RemovedItems = Enumerable.Empty<AbstractDockItem> ();
		}
		
		public ItemsChangedArgs (IEnumerable<AbstractDockItem> addedItems, IEnumerable<AbstractDockItem> removedItems)
		{
			AddedItems = addedItems == null ? Enumerable.Empty<AbstractDockItem> () : addedItems;
			RemovedItems = removedItems == null ? Enumerable.Empty<AbstractDockItem> () : removedItems;
		}
	}
}
