//  
//  Copyright (C) 2010 Robert Dyer
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using System;

using Gdk;

using Docky.CairoHelper;
using Docky.Items;

namespace Docky.Menus
{
	class ProxyMenuItem : MenuItem
	{
		const int ICON_SIZE = 16;
		
		AbstractDockItem item;

		public ProxyMenuItem (AbstractDockItem item) : base ()
		{
			this.item = item;
			
			Bold = false;
			Text = item.HoverText;
			Disabled = false;
			
			using (DockySurface surface = new DockySurface (ICON_SIZE, ICON_SIZE)) {
				ForcePixbuf = item.IconSurface (surface, ICON_SIZE, ICON_SIZE, 0).LoadToPixbuf ();
			}
			
			Clicked += delegate {
				this.item.Clicked (1, Gdk.ModifierType.None, 0, 0);
			};
		}
	}
}
